/* This file is part of Eclat.
   Copyright (C) 2012-2015 Sergey Poznyakoff.
 
   Eclat is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.
 
   Eclat is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with Eclat.  If not, see <http://www.gnu.org/licenses/>. */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#ifdef HAVE_GETOPT_H
# include <getopt.h>
#endif
#include <errno.h>
#include <sysexits.h>
#include <libeclat.h>
#include <json.h>

void
usage()
{
	printf("usage: %s [OPTIONS] FILE [INPUT]\n", program_name);
}

static char *
readfile(FILE *fp)
{
	char inbuf[4096];
	char *buf;
	size_t bufsize = 0;
	size_t bufmax;
	size_t n;

	bufmax = sizeof(inbuf);
	buf = grecs_malloc(bufmax);
	
	while (n = fread(inbuf, 1, sizeof(inbuf), fp)) {
		if (bufsize + n >= bufmax) {
			bufmax *= 2;
			buf = grecs_realloc(buf, bufmax);
		}
		memcpy(buf + bufsize, inbuf, n);
		bufsize += n;
	}
	if (ferror(fp))
		die(EX_NOINPUT, "read error");
	if (bufsize + 1 >= bufmax) {
		bufmax++;
		buf = grecs_realloc(buf, bufmax);
	}
	buf[bufsize] = 0;
	return buf;
}
	


int
main(int argc, char **argv)
{
	char *file = "-";
	FILE *fp;
	int rc;
	char *input;
	struct json_object *obj;
	
	set_program_name(argv[0]);
	while ((rc = getopt(argc, argv, "f:h")) != EOF) {
		switch (rc) {
		case 'f':
			file = optarg;
			break;
		case 'h':
			usage();
			return 0;
		default:
			exit(EX_USAGE);
		}
	}
	argc -= optind;
	argv += optind;
	
	//FIXME: process args

	if (strcmp(file, "-") == 0)
		fp = stdin;
	else {
		fp = fopen(file, "r");
		if (!fp)
			die(EX_NOINPUT, "cannot open input file \"%s\": %s",
			    file, strerror(errno));
	}
	
	input = readfile(fp);
	obj = json_parse_string(input, strlen(input));
	if (!obj)
		die(EX_DATAERR,
		    "%s: near %s",
		    json_err_diag,
		    json_err_ptr);
	
	json_object_free(obj);
	return 0;
}

	
